# -*- coding: utf-8 -*-
import pytest
import requests

from kawasemi.backends.twitter import TwitterChannel
from kawasemi.exceptions import HttpError


@pytest.fixture()
def channel():
    config = {
        "_backend": "kawasemi.backends.twitter.TwitterChannel",
        "api_key": "api_key",
        "api_secret": "api_secret",
        "access_token": "access_token",
        "access_token_secret": "access_token_secret"
    }
    return TwitterChannel(**config)


class TestTwitterChannel(object):
    def test_send(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.ok
        post.return_value = response

        channel.send("First tweet!")

    def test_send_fail_invalid_token(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.unauthorized
        post.return_value = response

        with pytest.raises(HttpError):
            channel.send("Test tweet", fail_silently=False)

        channel.send("Test tweet", fail_silently=True)



# -*- coding: utf-8 -*-
from copy import deepcopy

import pytest
import requests

from kawasemi.backends.hipchat import HipChatChannel
from kawasemi.exceptions import HttpError, ImproperlyConfigured


config =  {
    "_backend": "kawasemi.backends.hipchat.HipChatChannel",
    # Required
    "api_id": "api_id",
    "token": "token"
}


@pytest.fixture()
def channel():
    return HipChatChannel(**config)


class TestHipChatChannel(object):
    def test_init(self):
        with pytest.raises(TypeError):
            HipChatChannel(**{})

        with pytest.raises(ImproperlyConfigured):
            conf = deepcopy(config)
            conf["color"] = "blue"
            HipChatChannel(**conf)

        with pytest.raises(ImproperlyConfigured):
            conf = deepcopy(config)
            conf["notify"] = "true"
            HipChatChannel(**conf)

    def test_send(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.no_content
        post.return_value = response

        channel.send("Test message")

        channel.send("Test message with `color=green`.\n"
                     "https://www.hipchat.com/",
                     options={"hipchat": {"color": "green"}})

        channel.send("Test message with `message_format=text`.\n"
                     "https://www.hipchat.com/",
                     options={"hipchat": {"message_format": "text"}})

    def test_send_fail_invalid_token(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.bad_request
        post.return_value = response

        with pytest.raises(HttpError):
            channel.send("Test message", fail_silently=False)

        channel.send("Test message", fail_silently=True)



# # -*- coding: utf-8 -*-
import pytest
import requests

from kawasemi.backends.yo import YoChannel
from kawasemi.exceptions import HttpError


@pytest.fixture()
def channel():
    config = {
        "_backend": "kawasemi.backends.yo.YoChannel",
        "api_token": "api_token",
    }
    return YoChannel(**config)


class TestYoChannel(object):
    def test_send(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.ok
        post.return_value = response

        # Just Yo
        channel.send(None)

        # Yo with text
        channel.send(u"🍣")

        # Yo Link
        channel.send(None, options={
            "yo": {"link": "http://docs.justyo.co/v1.0/docs/yo"}})

        # Yo Location
        channel.send(None, options={
            "yo": {"location": "35.0261581,135.7818476"}})

    def test_send_fail(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.forbidden
        post.return_value = response

        with pytest.raises(HttpError):
            channel.send(None, fail_silently=False)

        channel.send(None, fail_silently=True)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import sys
from typing import Dict, Optional, Text  # noqa: F401

from .__about__ import __version__
from .backends.base import BaseChannel  # noqa: F401
from .exceptions import ImproperlyConfigured
from .types import SendOptions  # noqa: F401

__all__ = ["Kawasemi", "__version__"]


class Kawasemi(object):
    def __init__(self, settings):
        self.settings = settings
        self._backends = {}  # type: Dict[str, BaseChannel]

    def _load_module(self, name):
        # type: (str) -> BaseChannel
        __import__(name)
        return sys.modules[name]

    def _load_backend(self, name):
        # type: (str) -> BaseChannel
        try:
            return self._backends[name]
        except KeyError:
            module_name, klass_name = name.rsplit(".", 1)
            module = self._load_module(str(module_name))
            self._backends[name] = getattr(module, klass_name)
            return self._backends[name]

    def send(self, message, channel_name=None, fail_silently=False,
             options=None):
        # type: (Text, Optional[str], bool, Optional[SendOptions]) -> None
        """Send a notification to channels

        :param message: A message
        """
        if channel_name is None:
            channels = self.settings["CHANNELS"]
        else:
            try:
                channels = {
                    "__selected__": self.settings["CHANNELS"][channel_name]
                }
            except KeyError:
                raise Exception("channels does not exist %s", channel_name)

        for _, config in channels.items():
            if "_backend" not in config:
                raise ImproperlyConfigured(
                    "Specify the backend class in the channel configuration")

            channel = self._load_backend(config["_backend"])(**config)
            channel.send(message, fail_silently=fail_silently, options=options)



# -*- coding: utf-8 -*-
import pytest

from kawasemi.backends.base import BaseChannel


class TestBaseChannel(object):
    def test_init(self):
        with pytest.raises(TypeError):
            BaseChannel()

    def test_set_payload_from_options(self):
        payload = {}
        options = {
            "test": {
                "key1": "value1",
                "key2": "value2",
                "key3": "value3"
            }
        }
        payload_keys = ["key1", "key2"]

        BaseChannel._set_payload_from_options(
            payload, options, "test", payload_keys)

        assert len(payload) == len(payload_keys)
        for key in payload:
            assert key in payload_keys
            assert payload[key] == options["test"][key]



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from kawasemi import Kawasemi


def _load_django_settings():
    from django.conf import settings
    return settings.KAWASEMI


_sender = Kawasemi(_load_django_settings())
send = _sender.send



# -*- coding: utf-8 -*-
from copy import deepcopy

import pytest
import requests

from kawasemi.backends.slack import SlackChannel
from kawasemi.exceptions import HttpError, ImproperlyConfigured


config = {
    "_backend": "kawasemi.backends.slack.SlackChannel",
    "url": "url"
}


@pytest.fixture()
def channel():
    return SlackChannel(**config)


class TestSlackChannel(object):
    def test_init(self):
        with pytest.raises(ImproperlyConfigured):
            conf = deepcopy(config)
            conf["icon_emoji"] = ":+1:"
            conf["icon_url"] = "http://www.example.com/"
            SlackChannel(**conf)

    def test_send(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.ok
        post.return_value = response

        channel.send("Test message.\nhttps://slack.com/")

        channel.send("Test message. `unfurl_links=True`\n"
                          "https://slack.com/",
                          options={"slack": {"unfurl_links": True}})

        channel.send("Test message with attachments", options={
            "slack": {
                "attachments": [
                    {
                        "fallback": "Attachment 1",
                        "text": "Attachment 1",
                        "color": "#36a64f"
                    },
                    {
                        "fallback": "Attachment 2",
                        "text": "Attachment 2",
                        "color": "warning",
                        "fields": [
                            {
                                "title": "Field 1",
                                "value": "Field 1 value",
                                "short": False
                            },
                            {
                                "title": "Field 2",
                                "value": "Field 2 value",
                                "short": True
                            },
                            {
                                "title": "Field 3",
                                "value": "Field 3 value",
                                "short": True
                            }
                        ]
                    }
                ]
            }
        })

    def test_send_fail_invalid_url(self, channel, mocker):
        post = mocker.patch("requests.post")
        response = requests.Response()
        response.status_code = requests.codes.forbidden
        post.return_value = response

        with pytest.raises(HttpError):
            channel.send("Test message", fail_silently=False)

        channel.send("Test message", fail_silently=True)



SECRET_KEY = "fake-secret-key"

INSTALLED_APPS = [
    "tests",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

MIDDLEWARE_CLASSES = []

KAWASEMI = {
    "CHANNELS": {
        "hipchat": {
            "_backend": "kawasemi.backends.hipchat.HipChatChannel",
            # Required
            "api_id": "api_id",
            "token": "token",
            # Optional
            "base_url": "base_url",
            "color": "red",
            "notify": True
        },
        "slack": {
            "_backend": "kawasemi.backends.slack.SlackChannel",
            # Required
            "url": "url",
            # Optional
            "username": "username",
            "icon_emoji": ":smile:",
            "channel": "channel"
        },
        "twitter": {
            "_backend": "kawasemi.backends.twitter.TwitterChannel",
            # Required
            "api_key": "api_key",
            "api_secret": "api_secret",
            "access_token": "access_token",
            "access_token_secret": "access_token_secret"
        },
        "yo": {
            "_backend": "kawasemi.backends.yo.YoChannel",
            # Required
            "api_token": "api_token",
            # Optional
            "username": "username"
        }
    }
}



# -*- coding: utf-8 -*-
from django.test import TestCase

import kawasemi.django


class DjangoModuleTestCase(TestCase):
    def test_load_django_settings(self):
        settings = kawasemi.django._load_django_settings()
        from .settings import KAWASEMI
        self.assertDictEqual(settings, KAWASEMI)



# -*- coding: utf-8 -*-
import pytest

from kawasemi import Kawasemi


class TestKawasemi(object):
    def test_load_module(self):
        kawasemi = Kawasemi({})
        from kawasemi.backends import slack
        module_name = "kawasemi.backends.slack"
        assert kawasemi._load_module(module_name) == slack

    def test_load_backend(self):
        kawasemi = Kawasemi({})
        from kawasemi.backends.slack import SlackChannel
        klass_name = "kawasemi.backends.slack.SlackChannel"
        assert kawasemi._load_backend(klass_name) == SlackChannel

    def test_send_channel_not_exist(self):
        kawasemi = Kawasemi({
            "CHANNELS": {}
        })

        with pytest.raises(Exception):
            kawasemi.send("message", channel_name="not_exists",
                          fail_silently=False, options=None)

    def test__send_all_channels(self, mocker):
        """test case for channel_name=None"""
        settings = {
            "CHANNELS": {
                "channel1": {
                    "_backend": "kawasemi.backends.hipchat.HipChatChannel",
                    "api_id": "api_id",
                    "token": "token"
                },
                "channel2": {
                    "_backend": "kawasemi.backends.slack.SlackChannel",
                    "url": "url"
                }
            }
        }
        kawasemi = Kawasemi(settings)
        lb = mocker.patch.object(kawasemi, "_load_backend")
        kawasemi.send("message", channel_name=None, fail_silently=False,
                      options=None)
        assert lb.call_count == len(settings["CHANNELS"])
        lb.assert_any_call(settings["CHANNELS"]["channel1"]["_backend"])
        lb.assert_any_call(settings["CHANNELS"]["channel2"]["_backend"])

    def test__send_specific_channel(self, mocker):
        """test case for channel_name != None"""
        settings = {
            "CHANNELS": {
                "channel1": {
                    "_backend": "kawasemi.backends.hipchat.HipChatChannel",
                    "api_id": "api_id",
                    "token": "token"
                },
                "channel2": {
                    "_backend": "kawasemi.backends.slack.SlackChannel",
                    "url": "url"
                }
            }
        }
        kawasemi = Kawasemi(settings)
        lb = mocker.patch.object(kawasemi, "_load_backend")
        kawasemi.send("message", channel_name="channel1", fail_silently=False,
                      options=None)
        lb.assert_called_once_with(
            settings["CHANNELS"]["channel1"]["_backend"])



#!/usr/bin/env python
from codecs import open
from os import path
import sys

from setuptools import find_packages, setup


here = path.abspath(path.dirname(__file__))

def get_version():
    about_file = path.join(here, 'kawasemi/__about__.py')
    if sys.version_info[0] >= 3:
        about = {}
        with open(about_file) as fp:
            exec(fp.read(), about)
        return about['__version__']
    else:
        execfile(about_file)
        return __version__

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

requires = [
    'requests>=2.7.0',
    'requests-oauthlib>=0.5.0',
    'six>=1.9.0'
]

extras_require = {
    ':python_version < "3.5"': ['typing'],
    'docs': [
        'Sphinx>=1.5,<1.6',
        'sphinx-rtd-theme>=0.2,<0.3'
    ],
    'test': [
        'coverage>=4.3.4,<5.0.0',
        'coveralls>=1.1,<2.0',
        'flake8>=3.3.0,<4.0.0',
        'pytest>=3.0.0,<4.0.0',
        'pytest-cov>=2.4.0,<3.0.0',
        'pytest-mock>=1.5.0,<2.0.0',
    ],
    'test:python_version >= "3.3"': [
        'mypy-lang>=0.4.6,<0.5.0',
    ],
}

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Internet :: WWW/HTTP',
]

setup(
    name='kawasemi',
    version=get_version(),
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    include_package_data=True,
    license='MIT',
    description='A Python library for sending notifications to services such as Slack, HipChat, Twitter, and so on',
    long_description=long_description,
    url='https://github.com/ymyzk/kawasemi',
    author='Yusuke Miyazaki',
    author_email='miyazaki.dev@gmail.com',
    install_requires=requires,
    extras_require=extras_require,
    classifiers=classifiers
)
